unit ChoiceMainForm;

interface

uses
  packages.tools.VersionInformation,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, Buttons, mcSamplingThread, mcSamplingResult;

type
  TfrmChoiceMainForm = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Label1: TLabel;
    Panel3: TPanel;
    Panel4: TPanel;
    btnViewProcesses: TBitBtn;
    btnViewResults: TBitBtn;
    btnStartProfiling: TBitBtn;
    bLoad: TButton;
    procedure bLoadClick(Sender: TObject);
    procedure btnStartProfilingClick(Sender: TObject);
    procedure btnViewResultsClick(Sender: TObject);
    procedure btnViewProcessesClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    fInfoRecord: TExtractFileVersionData;

  protected
    procedure LiveViewClicked(aProcessId: Integer);
    procedure ProfileItClicked(aProcessId: Integer);
    procedure ShowResultsClicked(aSamplingThread: TSamplingThread; aSamplingResult: TSamplingResult = nil);
  public
    { Public declarations }
  end;

  // returning Version information from gitversion.inc
  function GetVersionInformation: TExtractFileVersionData;
  function GetVersionNumber: String;


var
  frmChoiceMainForm: TfrmChoiceMainForm;

implementation

uses
  ProfilingFrame, BaseActionForm, ResultsForm, ProcessesFrame, mcProcessSampler, LiveViewFrame,
  uDebugUtils, LoadScreenFrame;

{$R *.dfm}

function GetVersionInformation: TExtractFileVersionData;
begin
  Result := VersionInformation.ExtractInformations;
end;

function GetVersionNumber: string;
begin
  Result := GetVersionInformation.FileVersionData.FileVersionNr;
end;


procedure TfrmChoiceMainForm.bLoadClick(Sender: TObject);
var
  vLoad: ILoadProcess;
  i: Integer;
begin
  vLoad := TLoadingProcess.Create('testding', 300);
  for i := 0 to 300 do
  begin
    vLoad.Value := i;
    Sleep(15);

  end;
  Close;
end;

procedure TfrmChoiceMainForm.btnStartProfilingClick(Sender: TObject);
var fram: TframProfiling;
begin
  fram := TframProfiling.Create(nil);
  fram.Name := fram.Name + IntToHex(Integer(fram),8);

  fram.OnResultsClick   := Self.ShowResultsClicked;

  TfrmAction.ShowFormWithFrame(fram, Format('Profiling',[]));
end;

procedure TfrmChoiceMainForm.btnViewProcessesClick(Sender: TObject);
var fram: TframProcesses;
begin
  fram := TframProcesses.Create(nil);
  fram.Name := fram.Name + IntToHex(Integer(fram),8);

  fram.OnLiveViewClick  := Self.LiveViewClicked;
  fram.OnProfileItClick := Self.ProfileItClicked;

  TfrmAction.ShowFormWithFrame(fram, Format('View running processes',[]));
end;

procedure TfrmChoiceMainForm.LiveViewClicked(aProcessId: Integer);
var fram: TframLiveView;
begin
  fram := TframLiveView.Create(nil);
  fram.Name := fram.Name + IntToHex(Integer(fram),8);

  if aProcessId > 0 then
    fram.ProcessObject  := TProcessSampler.Create(aProcessId);
  fram.OnProfileItClick := Self.ProfileItClicked;

  TfrmAction.ShowFormWithFrame(fram, Format('Live View (PID: %d)',[aProcessId]));
end;

procedure TfrmChoiceMainForm.ProfileItClicked(aProcessId: Integer);
var fram: TframProfiling;
begin
  fram := TframProfiling.Create(nil);
  fram.Name := fram.Name + IntToHex(Integer(fram),8);

  fram.OnResultsClick   := Self.ShowResultsClicked;
  if aProcessId > 0 then
    //create new object for profiling
    fram.ProcessObject := TProcessSampler.Create(aProcessId);

  TfrmAction.ShowFormWithFrame(fram, Format('Profiling (PID: %d)',[aProcessId]));
end;

procedure TfrmChoiceMainForm.btnViewResultsClick(Sender: TObject);
begin
  ShowResultsClicked(nil);
end;

procedure TfrmChoiceMainForm.FormCreate(Sender: TObject);
var
  vSourceBase: string;
  vCaption: string;
begin
  uDebugUtils.GetDebugPrivs;
  //EnableThreadPrivilege(True, 'SeDebugPrivilege');

  vSourceBase := ' | OpenSource Base: V1.1.6';
  fInfoRecord := VersionInformation.ExtractInformations;

  {$I gitversion.inc}

  // dont use the common way. has its own information. Its open source from guthup. see readme!
  // if VersionInformation.IsRelease then
{$ifdef DEBUG }
  Self.Caption := Self.Caption + ' - ' + 'Entwicklerversion' + ' | ' + GIT_CURRENT_BRANCH + ' | ' + GIT_AUTHOR_NAME + vSourceBase
{$ELSE}
  Self.Caption := Self.Caption + vSourceBase;
{$endif }
end;

procedure TfrmChoiceMainForm.ShowResultsClicked(aSamplingThread: TSamplingThread; aSamplingResult: TSamplingResult = nil);
var frm: TfrmSamplingResults;
begin
  frm := TfrmSamplingResults.Create(Self);
  frm.Name := frm.Name + IntToHex(Integer(frm),8);

  if aSamplingResult <> nil then
    frm.SamplingResult := aSamplingResult
  else if aSamplingThread <> nil then
    frm.ProcessObject := aSamplingThread.ProcessObject;

  frm.Position := poDesigned;
  frm.Top  := Screen.ActiveCustomForm.Top + 25;
  frm.Left := Screen.ActiveCustomForm.Left + 25;

  frm.Show;
end;

end.
