unit LoadScreenFrame;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ComCtrls, Vcl.StdCtrls, Vcl.ExtCtrls;

type
  TLoadscreen = class(TForm)
    pbProgress: TProgressBar;
    lTop: TLabel;
    lMiddle: TLabel;
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

  ILoadProcess = interface
    procedure CurrentValue(const aCurrentValue: Cardinal);
    property Value: Cardinal write CurrentValue;
  end;

  TLoadingProcess = class (TInterfacedObject, ILoadProcess)
  private
    fFrame: TLoadscreen;
    fStartTime: TDateTime;
    fSuccessValue: Cardinal;
    fWorkName: string;
  public
    constructor Create(const aSuccessValue: Cardinal); overload;
    class function Create(const aHeading: string; const aSuccessValue: Cardinal):
        TLoadingProcess; overload;
    destructor Destroy; override;

    procedure CurrentValue(const aCurrentValue: Cardinal);
  end;

implementation

uses
  System.DateUtils;

{$R *.dfm}

{ TLoadingProcess }

constructor TLoadingProcess.Create(const aSuccessValue: Cardinal);
begin
  fSuccessValue := aSuccessValue;
  inherited Create();
  fFrame := TLoadscreen.Create(nil);
  fFrame.pbProgress.Max := aSuccessValue;
  fFrame.Position := poScreenCenter;
  fFrame.Show;
  fFrame.BringToFront;
  fStartTime := Now();
  fWorkName := 'loading';
end;

class function TLoadingProcess.Create(const aHeading: string; const
    aSuccessValue: Cardinal): TLoadingProcess;
begin
  Result := Create(aSuccessValue);
//  Result.fFrame.Caption := aHeading;
  Result.fWorkName := aHeading;
end;

procedure TLoadingProcess.CurrentValue(const aCurrentValue: Cardinal);
begin
  TThread.Synchronize(nil, procedure
    begin
      fFrame.lTop.Caption := Format('%s ... %3.2f%%', [fWorkName, aCurrentValue / fSuccessValue * 100]);
      fFrame.lMiddle.Caption := Format('elapsed time: %12.3n s', [SecondSpan(Now(), fStartTime)]);
      fFrame.pbProgress.Position := aCurrentValue;
      Application.ProcessMessages;
    end);
end;

destructor TLoadingProcess.Destroy;
begin
  fFrame.Free;
  inherited;
end;

end.
