unit ResultsDetailedViewFrame;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, BaseTreeFrame, VirtualTrees,
  mcSamplingResult, uResultTypes,
  StdCtrls, ExtCtrls, FunctionListFrame, Menus;

type
  TframDetailedView = class(TframBaseTree)
    Splitter1: TSplitter;
    gbParents: TGroupBox;
    gbChildren: TGroupBox;
    Splitter2: TSplitter;
    framFunctionListChilds: TframFunctionList;
    framFunctionListParents: TframFunctionList;
    procedure framFunctionListChildsvtreeItemsKeyDown(Sender: TObject; var Key:
        Word; Shift: TShiftState);
    procedure framFunctionListParentsvtreeItemsKeyDown(Sender: TObject; var Key:
        Word; Shift: TShiftState);
    procedure vtreeItemsInitNode(Sender: TBaseVirtualTree; ParentNode, Node: PVirtualNode;
      var InitialStates: TVirtualNodeInitStates);
    procedure vtreeItemsGetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex;
      TextType: TVSTTextType; var CellText: string);
    procedure vtreeItemsChange(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure vtreeItemsColumnResize(Sender: TVTHeader; Column: TColumnIndex);
    procedure vtreeItemsExpanded(Sender: TBaseVirtualTree; Node: PVirtualNode);
  strict private
    procedure doOnDblClickNode(const aSelected: TSubFunction);
  private
    FThreadItem: TThreadResult;
    procedure SetThreadItem(const Value: TThreadResult);
  protected
  public
    procedure AfterConstruction;override;
    procedure FocusFunction(aTargetFunction: TFlatFunction);
    function GetSelectedFlatFunction: TFlatFunction;

    property ThreadItem: TThreadResult read FThreadItem write SetThreadItem;
  end;

implementation

{$R *.dfm}

type
  // This is a very simple record we use to store data in the nodes.
  // Since the application is responsible to manage all data including the node's caption
  // this record can be considered as minimal requirement in all VT applications.
  // Extend it to whatever your application needs.
  PMyRec = ^TMyRec;
  TMyRec = record
    ModuleRec   : TModule;
    UnitRec     : TUnit;
    FunctionRec : TFlatFunction;
    LineRec     : TFunctionLine;
    TotalProcent, TotalChildProcent: Single;
  end;

{todo
  show red colors
  full expand node if one child
  expand modules 1 level at init
  parent + child calls
}

procedure IncF(var aFloat: Real; aIncrement: Real);
begin
  aFloat := aFloat + aIncrement;
end;

  { TframBaseTree1 }

procedure TframDetailedView.AfterConstruction;
begin
  inherited;
  // Let the tree know how much data space we need.
  vtreeItems.NodeDataSize := SizeOf(TMyRec);
  // Set an initial number of nodes.
  vtreeItems.RootNodeCount := 0;
end;

procedure TframDetailedView.SetThreadItem(const Value: TThreadResult);
begin
  FThreadItem := Value;

  vtreeItems.Clear;
  vtreeItems.RootNodeCount := FThreadItem.ModuleResult.Count;

  //sort on calls
  vtreeItems.SortTree(4, sdDescending);
end;

procedure TframDetailedView.vtreeItemsChange(Sender: TBaseVirtualTree; Node: PVirtualNode);
var
  Data: PMyRec;
begin
  //function
  if Sender.GetNodeLevel(Node) = 2 then
  begin
    Data := Sender.GetNodeData(Node);

//    framFunctionListChilds.SampleCount    := Self.ThreadItem.SampleCount;
//    framFunctionListParents.SampleCount   := Self.ThreadItem.SampleCount;
    framFunctionListChilds.Duration    := Self.ThreadItem.Duration;
    framFunctionListParents.Duration   := Self.ThreadItem.Duration;

    framFunctionListChilds.FunctionItems  := Data.FunctionRec.ChildCalls;
    framFunctionListChilds.OnDblClickNode := doOnDblClickNode;
    framFunctionListParents.FunctionItems := Data.FunctionRec.ParentCalls;
    framFunctionListParents.OnDblClickNode := doOnDblClickNode;
    gbParents.Caption := Format('Parents of ''%s''', [Data.FunctionRec.CompleteName]);
  end;
end;

procedure TframDetailedView.doOnDblClickNode(const aSelected: TSubFunction);
begin
  FocusFunction(aSelected.FunctionRef);
end;

procedure TframDetailedView.FocusFunction(aTargetFunction: TFlatFunction);
  type TFindNode = reference to function(const aNode: PVirtualNode): Boolean;
  function findInChildNodes(const aStartNode: PVirtualNode; aPredicat: TFindNode): PVirtualNode;
  var
    Child: PVirtualNode;
    i: Integer;
  begin
    Result := nil;
    if not Assigned(aStartNode) or (aStartNode.ChildCount = 0) then
      Exit();
    Child := aStartNode.FirstChild;
    while Assigned(Child) do
    begin
      if aPredicat(Child) then
        Exit(Child);
      Child := Child.NextSibling;
    end;
  end;
var
  Node: PVirtualNode;
  Data: PMyRec;
begin
  // find module
  Node := findInChildNodes(vtreeItems.RootNode, function (const aNode: PVirtualNode): Boolean
    begin
      Data := vtreeItems.GetNodeData(aNode);
      Result := SameStr(Data.ModuleRec.ModuleName, aTargetFunction.ModuleName);
    end);
  // find unit
  Node := findInChildNodes(Node, function (const aNode: PVirtualNode): Boolean
    begin
      Data := vtreeItems.GetNodeData(aNode);
      Result := SameStr( Data.UnitRec.UnitName, aTargetFunction.UnitName);
    end);
  // find function
  Node := findInChildNodes(Node, function (const aNode: PVirtualNode): Boolean
    begin
      Data := vtreeItems.GetNodeData(aNode);
      Result := SameStr( Data.FunctionRec.FunctionName, aTargetFunction.FunctionName);
    end);
  if Assigned(Node) then
  begin
    vtreeItems.Selected[Node] := True;
    vtreeItems.FocusedNode := Node;
    vtreeItems.ScrollIntoView(Node, True);
    while Assigned(Node.Parent) do
    begin
      Node.Parent.States := Node.Parent.States + [vsExpanded];
      Node := Node.Parent;
    end;
//    vtreeItems.SetFocus;
  end;
end;

procedure TframDetailedView.framFunctionListChildsvtreeItemsKeyDown(Sender:
    TObject; var Key: Word; Shift: TShiftState);
begin
  inherited;
  if Key = VK_RIGHT then
    framFunctionListChilds.vtreeItemsDblClick(Sender)
  else if Key = VK_LEFT then
    framFunctionListParents.vtreeItems.SetFocus
  else
    framFunctionListChilds.vtreeItemsKeyDown(Sender, Key, Shift);
end;

procedure TframDetailedView.framFunctionListParentsvtreeItemsKeyDown(Sender:
    TObject; var Key: Word; Shift: TShiftState);
begin
  inherited;
  if Key = VK_LEFT then
    framFunctionListParents.vtreeItemsDblClick(Sender)
  else if Key = VK_RIGHT then
    framFunctionListChilds.vtreeItems.SetFocus
  else
    framFunctionListParents.vtreeItemsKeyDown(Sender, Key, Shift);
end;

function TframDetailedView.GetSelectedFlatFunction: TFlatFunction;
var
  Data: PMyRec;
  Nodes: TNodeArray;
begin
  Result := nil;
  Nodes := vtreeItems.GetSortedSelection(True);
  if Length(Nodes) <= 0 then Exit();
  if vtreeItems.GetNodeLevel(Nodes[0]) <> 2 then Exit(); // only functions accepted
  Data := vtreeItems.GetNodeData(Nodes[0]);
  Result := Data.FunctionRec;
end;

procedure TframDetailedView.vtreeItemsColumnResize(Sender: TVTHeader; Column:
    TColumnIndex);
  procedure _changeWidth(aHeader: TVTHeader);
  begin
    if Column = 0 then
      aHeader.Columns.Items[Column+1].Width :=
        Sender.Columns.Items[Column].Width - aHeader.Columns.Items[Column].Width
    else
      aHeader.Columns.Items[Column+1].Width :=
        Sender.Columns.Items[Column].Width;
  end;
begin
  inherited;
  if framFunctionListParents.vtreeItems.Header.Columns.Count <= Column then
    Exit();
  _changeWidth(framFunctionListChilds.vtreeItems.Header);
  _changeWidth(framFunctionListParents.vtreeItems.Header);
end;

procedure TframDetailedView.vtreeItemsExpanded(Sender: TBaseVirtualTree; Node: PVirtualNode);
begin
  if Sender.ChildCount[Node] <= 1 then
    Sender.Expanded[Node.FirstChild] := True;
end;

procedure TframDetailedView.vtreeItemsGetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType; var CellText: string);
var
  Data: PMyRec;
begin
  try
  // A handler for the OnGetText event is always needed as it provides the tree with the string data to display.
  // Note that we are always using WideString.
  Data := Sender.GetNodeData(Node);
  if Assigned(Data) then
  begin
    case Column of
      2: CellText := Format('%3.2f',[Data.TotalProcent]);
      3: CellText := Format('%3.2f',[Data.TotalChildProcent]);
    end;

    if Sender.GetNodeLevel(Node) = 0 then
    case Column of
      0: CellText := ExtractFileName(Data.ModuleRec.ModuleName);
      1: CellText := IntToStr(Data.ModuleRec.Calls);
      4: CellText := Format('%4.2fms',[Data.ModuleRec.TotalOwnTime]);
      5: CellText := Format('%4.2fms',[Data.ModuleRec.TotalChildTime]);
      6: CellText := Format('%4.2fms',[Data.ModuleRec.TotalTime]);
    end
    else if Sender.GetNodeLevel(Node) = 1 then
    case Column of
      0: CellText := ExtractFileName(Data.UnitRec.UnitName);
      1: CellText := IntToStr(Data.UnitRec.Calls);
      4: CellText := Format('%4.2fms',[Data.UnitRec.TotalOwnTime]);
      5: CellText := Format('%4.2fms',[Data.UnitRec.TotalChildTime]);
      6: CellText := Format('%4.2fms',[Data.UnitRec.TotalTime]);
    end
    else if Sender.GetNodeLevel(Node) = 2 then
    case Column of
      0: CellText := Data.FunctionRec.FunctionName;
      1: CellText := IntToStr(Data.FunctionRec.Count);
      4: CellText := Format('%4.2fms',[Data.FunctionRec.TotalOwnTime]);
      5: CellText := Format('%4.2fms',[Data.FunctionRec.TotalChildTime]);
      6: CellText := Format('%4.2fms',[Data.FunctionRec.TotalTime]);
    end
    else if Sender.GetNodeLevel(Node) = 3 then
    case Column of
      0: CellText := Data.LineRec.LineString;
      1: CellText := IntToStr(Data.LineRec.Count);
//      4: CellText := Format('%4.2fms',[Data.FunctionRec.TotalOwnTime]);
//      5: CellText := Format('%4.2fms',[Data.FunctionRec.TotalChildTime]);
      6: CellText := Format('%4.2fms',[Data.LineRec.Time]);
    else
      CellText := '';
    end;
  end;
  except
    Sleep(0);
  end;
end;

procedure TframDetailedView.vtreeItemsInitNode(Sender: TBaseVirtualTree; ParentNode, Node: PVirtualNode;
  var InitialStates: TVirtualNodeInitStates);
var
  ParentData,
  Data: PMyRec;
begin
  with Sender do
  begin
    Data := GetNodeData(Node);
    // Construct a node caption. This event is triggered once for each node but
    // appears asynchronously, which means when the node is displayed not when it is added.

    //MODULE
    if Sender.GetNodeLevel(Node) = 0 then
    begin
      if ThreadItem.ModuleResult.Count <= Integer(Node.Index) then exit;
      Data.ModuleRec          := TModule(ThreadItem.ModuleResult.Items[Node.Index]);
//      Data.TotalProcent       := (Data.ModuleRec.TotalOwnTime * 100) / ThreadItem.SampleCount;
//      Data.TotalChildProcent  := (Data.ModuleRec.TotalTime * 100)    / ThreadItem.SampleCount;
      Data.TotalProcent       := (Data.ModuleRec.TotalOwnTime * 100) / ThreadItem.Duration;
      Data.TotalChildProcent  := (Data.ModuleRec.TotalTime * 100)    / ThreadItem.Duration;
      ChildCount[Node]        := Data.ModuleRec.Units.Count;
    end
    //UNIT
    else if Sender.GetNodeLevel(Node) = 1 then
    begin
      ParentData := Sender.GetNodeData(Node.Parent);

      if (ParentData.ModuleRec.Units.Count <= Integer(Node.Index)) then exit;
      Data.UnitRec            := TUnit(ParentData.ModuleRec.Units.Items[Node.Index]);
      Data.TotalProcent       := (Data.UnitRec.TotalOwnTime * 100) / ThreadItem.Duration;
      Data.TotalChildProcent  := (Data.UnitRec.TotalTime * 100)    / ThreadItem.Duration;
      ChildCount[Node]        := Data.UnitRec.Functions.Count;
    end
    //FUNCTION
    else if Sender.GetNodeLevel(Node) = 2 then
    begin
      ParentData := Sender.GetNodeData(Node.Parent);

      if (ParentData.UnitRec.Functions.Count <= Integer(Node.Index)) then exit;
      Data.FunctionRec        := TFlatFunction(ParentData.UnitRec.Functions.Items[Node.Index]);
      Data.TotalProcent       := (Data.FunctionRec.TotalOwnTime * 100) / ThreadItem.Duration;
      Data.TotalChildProcent  := (Data.FunctionRec.TotalTime * 100)    / ThreadItem.Duration;
      ChildCount[Node]        := Data.FunctionRec.FunctionLines.Count;
    end
    //LINE
    else if Sender.GetNodeLevel(Node) = 3 then
    begin
      ParentData := Sender.GetNodeData(Node.Parent);

      if (ParentData.FunctionRec.FunctionLines.Count <= Integer(Node.Index)) then exit;
      Data.LineRec            := TFunctionLine(ParentData.FunctionRec.FunctionLines.Items[Node.Index]);
      Data.TotalProcent       := (Data.LineRec.Time * 100) / ThreadItem.Duration;
//      Data.TotalChildProcent  := (Data.LineRec.TotalTime * 100)    / ThreadItem.SampleCount;
//      ChildCount[Node]        := Length(Data.FunctionRec.FunctionLines);
    end;
  end;
end;

end.
