unit SearchFrunctionFrame;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls,
  System.Generics.Collections,
  uResultTypes;

type

  TframSearchFunction = class(TFrame)
    gbFunctionName: TGroupBox;
    eFunctionName: TEdit;
    gbResults: TGroupBox;
    lstSearchResults: TListBox;
    procedure eFunctionNameChange(Sender: TObject);
    procedure eFunctionNameKeyDown(Sender: TObject; var Key: Word; Shift:
        TShiftState);
  private
    fThreadListBox: TListBox;
    fThreadResults: TThreadResultList;
    fFunctions: TList<TFlatFunction>;
  public
    procedure AfterConstruction; override;
    procedure BeforeDestruction; override;
    property ThreadResults: TThreadResultList read fThreadResults write
        fThreadResults;
    function GetSelectedFlatFunction: TFlatFunction;
  end;

implementation

{$R *.dfm}

procedure TframSearchFunction.AfterConstruction;
begin
  inherited;
  fFunctions := TList<TFlatFunction>.Create();
end;

procedure TframSearchFunction.BeforeDestruction;
begin
  fFunctions.Clear;
  fFunctions.Free;
  inherited;
end;

procedure TframSearchFunction.eFunctionNameChange(Sender: TObject);
var
  vSearch: string;
  iResults: Integer;
  vTR: TThreadResult;
  iFunctions: Integer;
  vsl: TStringList;
begin
  fFunctions.Clear;
  lstSearchResults.Clear;
  vSearch := LowerCase(eFunctionName.Text);
  if vSearch.IsEmpty or not Assigned(ThreadResults) then
    Exit();
  for iResults := 0 to ThreadResults.Count-1 do
  begin
    vTR := ThreadResults.Items[iResults] as TThreadResult;
    if vTR.ThreadId = 0 then
      Continue;
    for iFunctions := 0 to vTR.FlatFunctions.Count - 1 do
    begin
      if vTR.FlatFunctions[iFunctions].FunctionName.ToLower.Contains(vSearch) then
      begin
        fFunctions.Add( vTR.FlatFunctions[iFunctions] );
        lstSearchResults.AddItem(Format('%6d   %s', [vTR.ThreadId, fFunctions.Last.FunctionName]), vTR);
      end;
    end;
  end;
end;

procedure TframSearchFunction.eFunctionNameKeyDown(Sender: TObject; var Key:
    Word; Shift: TShiftState);
begin
  if (Key = VK_DOWN) and (lstSearchResults.Items.Count > 0) then
  begin
    lstSearchResults.SetFocus;
    lstSearchResults.Selected[0] := True;
    lstSearchResults.OnClick(lstSearchResults);
  end;
end;

function TframSearchFunction.GetSelectedFlatFunction: TFlatFunction;
begin
  if lstSearchResults.ItemIndex < 0 then
    Exit(nil);
  Result := fFunctions[lstSearchResults.ItemIndex] as TFlatFunction;
end;

end.
